package dsexamples.misc;

import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import natinst.msl.datasocket.*;

/** DSReader demonstrates how to use the DataSocket library to read fom different
  * data sources.
  */
public class DSReader extends Applet implements WindowListener {
  Label sStatus, lab;
  TextField sURL;
  DataSocket ds1 = null;
  GraphCanvas grCanvas1 = null;
  Frame fr = null;

  // Currently, everything is done in init()
  public void init() {
    Choice choice = null;

    setLayout(new BorderLayout());
    Color cLabel = Color.lightGray;
    Color cText = Color.white;

    Panel URLPanel = new Panel();
    URLPanel.setLayout(new BorderLayout());
    URLPanel.add("West", lab = new Label("Source:", Label.LEFT));
    URLPanel.add("Center", sURL = new TextField("dstp://localhost/wave"));
    lab.setBackground(cLabel);
    sURL.setBackground(cText);

    Panel StatusPanel = new Panel();
    StatusPanel.setLayout(new BorderLayout());
    StatusPanel.add("West", lab = new Label("Status:", Label.LEFT));
    StatusPanel.add("Center", sStatus = new Label("Unconnected", Label.LEFT));
    lab.setBackground(cLabel);
    sStatus.setBackground(cLabel);

    Panel ChoicePanel = new Panel();
    ChoicePanel.setLayout(new BorderLayout());
    ChoicePanel.add("Center", choice = new Choice());
    choice.setBackground(cText);

    Panel p1 = new Panel();
    p1.setLayout(new GridLayout(3, 1));
    p1.add("North", URLPanel);
    p1.add("North", ChoicePanel);
    p1.add("North", StatusPanel);
    add("North", p1);

    Panel p = new Panel();
    p.setLayout(new GridLayout(4, 1));
    p.add(new Button("Connect"));
    p.add(new Button("Connect (Manual)"));
    p.add(new Button("Disconnect"));
    p.add(new Button("Update"));
    add("West", p);

    grCanvas1 = new GraphCanvas();
    add("Center", grCanvas1);

    choice.addItem("http://www.natinst.com/cworks/datasocket/chirp.wav");
    choice.addItem("http://www.natinst.com/cworks/datasocket/chirp.dsd");
    choice.addItem("ftp://ftp.natinst.com/support/compworks/datasocket/schirp.wav");
    choice.addItem("ftp://ftp.natinst.com/support/compworks/datasocket/tabtext.txt");
    choice.addItem("dstp://localhost/wave");
    choice.select("dstp://localhost/wave");

    ds1 = new DataSocket();
    ds1.addDSOnDataUpdatedListener(new DSOnDataUpdatedListener() {
      public void DSOnDataUpdated(DSOnDataUpdatedEvent e) {
        ds1_DSOnDataUpdated(e);
      }
    });

    ds1.addDSOnStatusUpdatedListener(new DSOnStatusUpdatedListener() {
      public void DSOnStatusUpdated(DSOnStatusUpdatedEvent e) {
        ds1_DSOnStatusUpdated(e);
      }
    });

  }

  // Plot new data as its available
  public void ds1_DSOnDataUpdated(DSOnDataUpdatedEvent e) {
    DSData cwData = ds1.getData();

    try {
      double[][] d = cwData.getValueAsDoubleArray2D();
      grCanvas1.plotY(d);
    } catch (DSDataException ex) {
      try {
        double[] d = cwData.getValueAsDoubleArray();
        grCanvas1.plotY(d);
      } catch (DSDataException ex2) {
        ex.printStackTrace();
      }
    }
  }

  // Show status
  public void ds1_DSOnStatusUpdated(DSOnStatusUpdatedEvent e) {
    sStatus.setText(ds1.getLastMessage());
  }

  // Respond to button presses, selection of choice
  public boolean action(Event evt, Object arg) {
	  if (evt.target instanceof Choice) {
	    sURL.setText(((Choice)evt.target).getSelectedItem());
	  } else {
	    if ("Connect".equals(arg)) {
		    ds1.connectTo(sURL.getText(), DSAccessModes.cwdsReadAutoUpdate);
	    } else if ("Disconnect".equals(arg)) {
		    ds1.disconnect();
	    } else if ("Update".equals(arg)) {
		    ds1.update();
      } else if ("Connect (Manual)".equals(arg)) {
        ds1.connectTo(sURL.getText(), DSAccessModes.cwdsRead);
	    }
	  }
	  return true;
  }

  public void setFrame(Frame fr1) {
    fr = fr1;
    fr.addWindowListener(this);
  }

  // close application when window close message is received
  public void windowClosing(WindowEvent e) {
   if (fr != null) {
      fr.setVisible(false);
      fr.dispose();
      System.exit(0);
    }
  }
  public void windowOpened(WindowEvent e) {}
  public void windowClosed(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}

  // Create frame and start DSReader Applet
  public static void main(String args[]) {
	  Frame f = new Frame("DSReader");

	  DSReader dsReader1 = new DSReader();
	  dsReader1.init();

    Color c = f.getForeground();
    dsReader1.setFrame(f);
    f.setBackground(c);
    f.add("Center", dsReader1);
    f.setSize(400, 200);
	  f.setVisible(true);

    dsReader1.start();
  }
}