package dsexamples.misc;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.applet.Applet;
import java.awt.event.*;
import natinst.msl.datasocket.*;

public class DSWeather extends Applet implements Runnable, WindowListener {

  /* Thread */
  Thread runner = null;

  /* Canvas and TCP/IP settings. */
  private Frame m_fr = null;
  private String m_dstpAddress = "dstp://weather.natinst.com/weather/";
  private DataSocket dsCurrent = new DataSocket();
  private DataSocket dsHistory = new DataSocket();
  GraphCanvas grHistory = new GraphCanvas();

  /* Theremometer parameters */
  private final int thermoFillColorRGB = 0xff00000;
  private final String thermoLabel = "Wind Direction (deg)";
  private final int thermoXCenter = 80;
  private final int thermoYCenter = 100;
  private final int thermoBarWidth = 10;
  private final int thermoBarHeight = 120;
  private float thermoMinValue = 0f;
  private float thermoMaxValue = 100f;
  private Image thermoBackgroundPicture;

  /* Hygrometer parameters */
  private final int hygroFillColorRGB = 0xff;
  private final int hygroXCenter = 70;
  private final int hygroYCenter = 100;
  private final int hygroBarWidth = 10;
  private final int hygroBarHeight = 140;
  private float hygroMinValue = 0f;
  private float hygroMaxValue = 100f;
  private Image hygroBackgroundPicture = thermoBackgroundPicture;

  /* Parameters common to all gauges */
  private final int gaugeHeight = 200;
  private final int gaugeWidth = 200;
  private final Color gaugeForegroundColor = Color.gray;
  private final Color gaugeBackgroundColor = Color.gray;
  private final boolean gaugeBorder = false;
  private final Color gaugeNeedleColorRGB = Color.red;

  /* Wind speed gauge parameters */
  //private final int speedGaugeNeedleLength = 70;
  private final float speedGaugeVertexXRatio = 0.5f;
  private final float speedGaugeVertexYRatio = 0.7f;
  private final float speedGaugeNeedleLengthRatio = 0.26f;
  private final String speedGaugeLabel = "Wind Speed (MPH)";
  private final int speedMinAngle = -60;
  private final int speedMaxAngle = 60;
  private final float speedGaugeMinValue = 0f;
  private final float speedGaugeMaxValue = 40f;
  private final float speedChartMinYValue = 0f;
	private final float speedChartMaxYValue = 40f;
  private Image speedGaugeBackgroundPicture;

  /* Wind direction gauge parameters */
  private final int directionGaugeNeedleLength = 55;
  private final float directionGaugeVertexXRatio = 0.5f;
  private final float directionGaugeVertexYRatio = 0.5f;
  private final float directionGaugeNeedleLengthRatio = 0.20f;
  private final String directionGaugeLabel = "Wind Direction";
  private final int directionMinAngle = -180;
  private final int directionMaxAngle = 180;
  private final float directionGaugeMinValue = 0f;
  private final float directionGaugeMaxValue = 360f;
  private Image directionGaugeBackgroundPicture;

  private float lvDirectionData;

  private final int sleepTime = 400; // msec of sleep time in the applet.

  // Declare weather instruments.
  Meter theThermometer;
  Meter theHygrometer;
  Gauge theSpeedGauge;
  Gauge theDirectionGauge;

  /***** METHODS (basically implementations of the Runnable interface!) *****/

  public void init()
  {
    Color cBackground = Color.lightGray;
    Color cLabel = Color.lightGray;
    setBackground(cBackground);

    // Load background images.
    try {
      speedGaugeBackgroundPicture = getImage(getCodeBase(), "images/speedbac.gif");
      thermoBackgroundPicture = getImage(getCodeBase(), "images/backgnd.gif");
      hygroBackgroundPicture = thermoBackgroundPicture;
      directionGaugeBackgroundPicture = getImage(getCodeBase(), "images/dirbac.gif");
    } catch (Exception e) {
      speedGaugeBackgroundPicture = Toolkit.getDefaultToolkit().getImage("images/speedbac.gif");
      thermoBackgroundPicture = Toolkit.getDefaultToolkit().getImage("images/backgnd.gif");
      hygroBackgroundPicture = thermoBackgroundPicture;
      directionGaugeBackgroundPicture = Toolkit.getDefaultToolkit().getImage("images/dirbac.gif");
    }

    Panel IndPanel = new Panel();
    IndPanel.setLayout(new GridLayout(1, 4));

    // Instantiate the weather instruments.  These are canvases.
    theThermometer = new Meter (thermoFillColorRGB,
				     											   thermoMinValue,
                            			   thermoMaxValue,
                                     "F",
                                     "Temperature",
                            			   thermoBackgroundPicture);

    theHygrometer = new Meter (hygroFillColorRGB,
				   										 		hygroMinValue,
                    				   		hygroMaxValue,
                                  "%",
                                  "Humidity",
                    				   		hygroBackgroundPicture);
    theHygrometer.setDrawBulb(false);

    theSpeedGauge = new Gauge (	gaugeForegroundColor,
			      										gaugeBackgroundColor,
			      										gaugeBorder,
			      										//gaugeNeedleColorRGB,
                                Color.green,
			      										speedGaugeNeedleLengthRatio,
                                speedGaugeVertexXRatio,
                                speedGaugeVertexYRatio,
			      										speedMinAngle,
			      										speedMaxAngle,
			      										speedChartMinYValue,
			      										speedChartMaxYValue,
			      										speedGaugeLabel,
			      										speedGaugeBackgroundPicture);

    theDirectionGauge = new Gauge (	gaugeForegroundColor,
                          				  gaugeBackgroundColor,
                          				  gaugeBorder,
                          				  gaugeNeedleColorRGB,
                                    directionGaugeNeedleLengthRatio,
                                    directionGaugeVertexXRatio,
                                    directionGaugeVertexYRatio,
                                    directionMinAngle,
                          				  directionMaxAngle,
                          				  directionGaugeMinValue,
                          				  directionGaugeMaxValue,
                                    directionGaugeLabel,
                          				  directionGaugeBackgroundPicture);

    IndPanel.add(theThermometer);
    IndPanel.add(theHygrometer);
    IndPanel.add(theSpeedGauge);
    IndPanel.add(theDirectionGauge);

    Panel graphPanel = new Panel();
    graphPanel.setLayout(new GridLayout(1, 3, 1, 0));

    // add new graphs
    grHistory.setAutoScale(false);
    grHistory.setYRange(0, 100);
    grHistory.setColor(0, Color.red);
    grHistory.setColor(1, Color.blue);
    grHistory.setColor(2, Color.green);
    grHistory.setLineWidth(3);
    graphPanel.add(grHistory);

    Panel gridPanel = new Panel();
    gridPanel.setLayout(new GridLayout(2, 1, 0, 1));

    gridPanel.add(IndPanel);
    gridPanel.add(graphPanel);

    Panel ChoicePanel = new Panel();
    ChoicePanel.setLayout(new GridLayout(1, 4));

    Label lab;
    Choice choice;
    ChoicePanel.add(lab = new Label("History:", Label.LEFT));
    lab.setBackground(cLabel);
    ChoicePanel.add(choice = new Choice());
    choice.add("1Minute");
    choice.add("10Minute");
    choice.add("1Hour");
    choice.add("8Hour");
    choice.add("24Hour");
    choice.add("48Hour");
    choice.select("1Hour");
    ChoicePanel.add(new Label(""));
    ChoicePanel.add(new Label(""));

    setLayout(new BorderLayout());
    add("Center", gridPanel);
    add("South", ChoicePanel);

  } // End init().

  public void start() {
    if (runner == null) {
      runner = new Thread(this);
      runner.start();
    }
  } // End start().

  public void stop() {
    if (runner != null) {
      runner.stop();
      runner = null;
    }
  } // End stop().

  public void run() {

    dsCurrent.addDSOnDataUpdatedListener(new DSOnDataUpdatedListener() {
        public void DSOnDataUpdated(DSOnDataUpdatedEvent e) {
          dsCurrent_DSOnDataUpdated(e);
        }
    });

    dsHistory.addDSOnDataUpdatedListener(new DSOnDataUpdatedListener() {
        public void DSOnDataUpdated(DSOnDataUpdatedEvent e) {
          dsHistory_DSOnDataUpdated(e);
        }
    });

    dsCurrent.connectTo(m_dstpAddress + "current", DSAccessModes.cwdsReadAutoUpdate);
    dsHistory.connectTo(m_dstpAddress + "1Hour", DSAccessModes.cwdsReadAutoUpdate);

  } // End run().

  private void dsCurrent_DSOnDataUpdated(DSOnDataUpdatedEvent e) {

    try {
      double[] d = dsCurrent.getData().getValueAsDoubleArray();

      float temp = (float)d[1];
      float windspeed = (float)d[2];
      float winddir = (float)d[3];
      float hum = (float)d[4];

      theDirectionGauge.updateValue(winddir);
      theSpeedGauge.updateValue(windspeed);
      theHygrometer.updateValue(hum);
      theThermometer.updateValue(temp);
    }
    catch (DSException ex) {
    }

  }

  // The history data
  private void dsHistory_DSOnDataUpdated(DSOnDataUpdatedEvent e) {

    // Something odd is going on with this data
    try {
      double[][] d = dsHistory.getData().getValueAsDoubleArray2D();

      // pull out temp, windspeed, and humidity
      double[][] pd = new double[3][d[0].length];
      pd[0] = d[1];  // temperature
      pd[1] = d[4];  // humidity
      pd[2] = d[2];  // windspeed

      // plot history values
      grHistory.plotY(pd);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  // Respond to button presses, selection of choice
  public boolean action(Event evt, Object arg) {
	  if (evt.target instanceof Choice) {
      String sText = ((Choice)evt.target).getSelectedItem();
      if ("Real Weather".equals(sText)) {

      }
      else if ("Pseudo Weather".equals(sText)) {

      }
      else {
         dsHistory.connectTo(m_dstpAddress + sText, DSAccessModes.cwdsReadAutoUpdate);
      }
	  }
	  return true;
  }

  // close application when window close message is received
  public void windowClosing(WindowEvent e) {
   if (m_fr != null) {
      m_fr.setVisible(false);
      m_fr.dispose();
      System.exit(0);
    }
  }
  public void windowOpened(WindowEvent e) {}
  public void windowClosed(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}

  public void setFrame(Frame f) {
    m_fr = f;
    m_fr.addWindowListener(this);
  }

  public static void main(String args[]) {
	  Frame f = new Frame("DSWeather");

	  DSWeather dsWeather1 = new DSWeather();
    try {
	     dsWeather1.init();
    } catch (Exception e) {
	     e.printStackTrace();
    }

    Color c = f.getForeground();
    dsWeather1.setFrame(f);
    f.setBackground(c);
    f.add("Center", dsWeather1);
    f.setSize(600, 350);
	  f.setVisible(true);

    dsWeather1.start();
  }

} // End DSWeather.

