package dsexamples.misc;

import java.awt.image.*;
import java.awt.*;
import java.io.*;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.applet.Applet;
import java.awt.event.*;

public class Gauge extends Canvas {

  private Color m_needle_color;
  private Color m_cLabel = Color.black;

  private Point m_vertex;
  private int m_needle_length;
  private int m_minAngle;
  private int m_maxAngle;
  private float m_minValue;
  private float m_maxValue;
  private String m_label;
  private float m_fXRatio, m_fYRatio;
  private float m_fnlr;

  private Image m_backgroundPicture;
  private Image m_offScreenImage;
  private Image m_scaledBackgroundPicture;
  private Graphics m_offScreenGraphics;

  private float m_value;

  private boolean m_bPainted = false;
  private boolean m_bPictLoaded = false;

   	/***** Constructor *****/
  Gauge	(	Color fg,
					Color bg,
					boolean bord,
					Color nc,
					float nl,
					float XRatio,
          float YRatio,
					int minAngle,
					int maxAngle,
					float minValue,
					float maxValue,
					String label,
					Image backgroundPicture)
  {
    m_needle_color = nc;
    m_fnlr = nl;
    m_fXRatio = XRatio;
    m_fYRatio = YRatio;
    m_minAngle = minAngle;
    m_maxAngle = maxAngle;
    m_minValue = minValue;
    m_maxValue = maxValue;
    m_label = label;
    m_backgroundPicture = backgroundPicture;
    enableEvents(AWTEvent.COMPONENT_EVENT_MASK);
  } // End constructor.

  /***** Methods *****/

  public void setLabelColor(Color c) { m_cLabel = c; }
  public Color getLabelColor() { return m_cLabel; }

  public void updateValue(float value)
  {
    m_value = value;
    if (!m_bPainted) {
      Resized();
    }
    if (m_bPainted) {
      PaintImage();
    }
    repaint();
  } // End updateGauge().

  private void Resized() {
    Rectangle rbounds = this.getBounds();

    if (rbounds.height != 0 && rbounds.width != 0) {

      m_bPictLoaded = false;
      m_scaledBackgroundPicture = m_backgroundPicture.getScaledInstance(rbounds.width,
                                          rbounds.height, Image.SCALE_FAST);

      m_vertex = new Point((int)(rbounds.width * m_fXRatio), (int)(rbounds.height * m_fYRatio));
      m_needle_length =  (int)(m_fnlr * Math.sqrt(rbounds.width * rbounds.width + rbounds.height * rbounds.height));
      m_offScreenImage = createImage(rbounds.width, rbounds.height);
      m_offScreenGraphics = m_offScreenImage.getGraphics();

      prepareImage(m_scaledBackgroundPicture, -1, -1, this);
      m_bPainted = true;
    }
  }

  protected void processComponentEvent(ComponentEvent e) {
    switch (e.getID()) {
      case ComponentEvent.COMPONENT_RESIZED:
        // if we resize, go ahead and recalculate image
        Resized();
        break;
      default: break;
    };
    super.processComponentEvent(e);
  }

  public void update(Graphics g)
  { paint(g);
  }

  public void PaintImage ()
  {
    float m;
    float rad_val;
    int deg_val;
    int x_dist, y_dist;
    int xLabelStart;
    int yLabelStart;

    Graphics g = m_offScreenGraphics;
    Rectangle rbounds = this.getBounds();

    // Draw the background picture.
    g.drawImage(m_scaledBackgroundPicture, 0, 0, null);

		Font f = new Font("Arial", Font.BOLD, 12);
		FontMetrics fontMetrics = getFontMetrics(f);

    // label
		xLabelStart = (m_vertex.x - (int)(fontMetrics.stringWidth(m_label)/2));
		yLabelStart = (int)(fontMetrics.getHeight() * 1.5);
		g.setColor(m_cLabel);
		g.drawString(m_label, xLabelStart, yLabelStart);

		m = (m_maxAngle - m_minAngle) / (m_maxValue - m_minValue);
		deg_val = (int)(m * (m_value - m_minValue)) + m_minAngle;

		g.setColor(m_needle_color);

		rad_val = (float)(deg_val * 3.14159 / 180);
    float fratio = rbounds.width / (rbounds.width + rbounds.height);
		x_dist = (int) (m_needle_length * Math.sin(rad_val));
		y_dist = - (int) (m_needle_length * Math.cos(rad_val));

		g.drawLine(m_vertex.x,m_vertex.y,m_vertex.x+x_dist,m_vertex.y+y_dist);
		g.drawLine(m_vertex.x-1,m_vertex.y,m_vertex.x+x_dist,m_vertex.y+y_dist);
		g.drawLine(m_vertex.x+1,m_vertex.y,m_vertex.x+x_dist,m_vertex.y+y_dist);
		g.drawLine(m_vertex.x,m_vertex.y-1,m_vertex.x+x_dist,m_vertex.y+y_dist);
		g.drawLine(m_vertex.x,m_vertex.y+1,m_vertex.x+x_dist,m_vertex.y+y_dist);

  } // End PaintImage().

  public synchronized boolean imageUpdate(Image image, int infoFlags,
        int x, int y, int width, int height)
  {
    if ((infoFlags & ImageObserver.ALLBITS) != 0) {
      m_bPictLoaded = true;
      PaintImage();
      repaint();
      return false;
    }
    else {
      return true;
    }
  }

  public void paint(Graphics g)
  {
    // draw offscreen image now
    if (m_bPictLoaded)
      g.drawImage(m_offScreenImage, 0, 0, this);
  } // End paint(g)

} // End Gauge.
